import { ssr, ssrHydrationKey, escape, createComponent, ssrAttribute, ssrSpread, renderToString as renderToString$1 } from 'solid-js/web';
import { createSignal, onMount, onCleanup, For, Show, createEffect, createContext, useContext } from 'solid-js';
import { closeBrackets, autocompletion } from '@codemirror/autocomplete';
import { StateField, RangeSet, Compartment, EditorState, SelectionRange, EditorSelection, RangeSetBuilder, Transaction } from '@codemirror/state';
import { vim } from '@replit/codemirror-vim';
import { EditorView, Decoration, WidgetType, ViewPlugin, keymap } from '@codemirror/view';
import { syntaxTree, syntaxHighlighting, HighlightStyle } from '@codemirror/language';
import { defaultKeymap, historyKeymap, indentMore, indentLess, history } from '@codemirror/commands';
import { markdown, markdownLanguage } from '@codemirror/lang-markdown';
import { languages } from '@codemirror/language-data';
import { tags } from '@lezer/highlight';

const HYDRATION_MARKER = "data-ink-mde-ssr-hydration-marker";
const HYDRATION_MARKER_SELECTOR = `[${HYDRATION_MARKER}]`;
const getHydrationMarkerProps = () => {
  {
    return {
      [HYDRATION_MARKER]: true
    };
  }
};

const types = {
  array: "[object Array]",
  object: "[object Object]",
  string: "[object String]",
  undefined: "[object Undefined]",
  window: "[object Window]"
};
const getType = (object) => {
  const type = Object.prototype.toString.call(object);
  if (type === types.object) {
    return `[object ${object.constructor.name}]`;
  }
};
const is = (object, type) => {
  return getType(object) === type;
};
const deepAssign = (target, source) => {
  const seen = /* @__PURE__ */ new WeakMap();
  const assign = (target2, source2) => {
    if (seen.get(target2))
      return target2;
    if (is(target2, types.object))
      seen.set(target2, true);
    if (is(source2, types.undefined))
      return target2;
    if (is(target2, types.array) && is(source2, types.array)) {
      return [...source2];
    }
    if (is(target2, types.object) && is(source2, types.object)) {
      return Object.keys(target2).reduce((replacement, key) => {
        if (Object.hasOwn(source2, key)) {
          replacement[key] = assign(target2[key], source2[key]);
        } else {
          replacement[key] = target2[key];
        }
        return replacement;
      }, {});
    }
    return source2;
  };
  return assign(target, source);
};
const override = (a, b) => {
  return deepAssign(a, b);
};

var Appearance = /* @__PURE__ */ ((Appearance2) => {
  Appearance2["Auto"] = "auto";
  Appearance2["Dark"] = "dark";
  Appearance2["Light"] = "light";
  return Appearance2;
})(Appearance || {});
var Extensions = /* @__PURE__ */ ((Extensions2) => {
  Extensions2["Appearance"] = "appearance";
  Extensions2["Attribution"] = "attribution";
  Extensions2["Autocomplete"] = "autocomplete";
  Extensions2["Images"] = "images";
  Extensions2["ReadOnly"] = "readonly";
  Extensions2["Spellcheck"] = "spellcheck";
  Extensions2["Vim"] = "vim";
  return Extensions2;
})(Extensions || {});
var Markup = /* @__PURE__ */ ((Markup2) => {
  Markup2["Bold"] = "bold";
  Markup2["Code"] = "code";
  Markup2["CodeBlock"] = "code_block";
  Markup2["Heading"] = "heading";
  Markup2["Image"] = "image";
  Markup2["Italic"] = "italic";
  Markup2["Link"] = "link";
  Markup2["List"] = "list";
  Markup2["OrderedList"] = "ordered_list";
  Markup2["Quote"] = "quote";
  Markup2["TaskList"] = "task_list";
  return Markup2;
})(Markup || {});
var PluginType = /* @__PURE__ */ ((PluginType2) => {
  PluginType2["Completion"] = "completion";
  PluginType2["Default"] = "default";
  PluginType2["Grammar"] = "grammar";
  return PluginType2;
})(PluginType || {});

const createElement = () => {
  {
    return {};
  }
};
const isAutoDark = () => {
  {
    return true;
  }
};
const isDark = (appearance) => {
  if (appearance === Appearance.Dark)
    return true;
  if (appearance === Appearance.Light)
    return false;
  return isAutoDark();
};
const makeVars = (state) => {
  const styles = [
    { suffix: "border-radius", default: "0.25rem" },
    { suffix: "color", default: "currentColor" },
    { suffix: "flex-direction", default: "column" },
    { suffix: "font-family", default: "inherit" },
    { suffix: "block-background-color", default: "#121212", light: "#f5f5f5" },
    { suffix: "block-background-color-on-hover", default: "#0f0f0f", light: "#e0e0e0" },
    { suffix: "block-max-height", default: "20rem" },
    { suffix: "block-padding", default: "0.5rem" },
    { suffix: "code-background-color", default: "var(--ink-internal-block-background-color)" },
    { suffix: "code-color", default: "inherit" },
    { suffix: "code-font-family", default: "'Monaco', Courier, monospace" },
    { suffix: "editor-font-size", default: "1em" },
    { suffix: "editor-line-height", default: "2em" },
    { suffix: "editor-padding", default: "0.5rem" },
    { suffix: "inline-padding", default: "0.125rem" },
    { suffix: "modal-position", default: "fixed" },
    { suffix: "syntax-atom-color", default: "#d19a66" },
    { suffix: "syntax-comment-color", default: "#abb2bf" },
    { suffix: "syntax-comment-font-style", default: "italic" },
    { suffix: "syntax-emphasis-color", default: "inherit" },
    { suffix: "syntax-emphasis-font-style", default: "italic" },
    { suffix: "syntax-hashtag-background-color", default: "#222", light: "#eee" },
    { suffix: "syntax-hashtag-color", default: "inherit" },
    { suffix: "syntax-heading-color", default: "inherit" },
    { suffix: "syntax-heading-font-weight", default: "600" },
    { suffix: "syntax-heading1-color", default: "var(--ink-internal-syntax-heading-color, inherit)" },
    { suffix: "syntax-heading1-font-size", default: "1.6em" },
    { suffix: "syntax-heading1-font-weight", default: "600" },
    { suffix: "syntax-heading2-color", default: "var(--ink-internal-syntax-heading-color, inherit)" },
    { suffix: "syntax-heading2-font-size", default: "1.5em" },
    { suffix: "syntax-heading2-font-weight", default: "600" },
    { suffix: "syntax-heading3-color", default: "var(--ink-internal-syntax-heading-color, inherit)" },
    { suffix: "syntax-heading3-font-size", default: "1.4em" },
    { suffix: "syntax-heading3-font-weight", default: "600" },
    { suffix: "syntax-heading4-color", default: "var(--ink-internal-syntax-heading-color, inherit)" },
    { suffix: "syntax-heading4-font-size", default: "1.3em" },
    { suffix: "syntax-heading4-font-weight", default: "600" },
    { suffix: "syntax-heading5-color", default: "var(--ink-internal-syntax-heading-color, inherit)" },
    { suffix: "syntax-heading5-font-size", default: "1.2em" },
    { suffix: "syntax-heading5-font-weight", default: "600" },
    { suffix: "syntax-heading6-color", default: "var(--ink-internal-syntax-heading-color, inherit)" },
    { suffix: "syntax-heading6-font-size", default: "1.1em" },
    { suffix: "syntax-heading6-font-weight", default: "600" },
    { suffix: "syntax-highlight-background-color", default: "#555555" },
    { suffix: "syntax-keyword-color", default: "#c678dd" },
    { suffix: "syntax-link-color", default: "inherit" },
    { suffix: "syntax-meta-color", default: "#abb2bf" },
    { suffix: "syntax-monospace-color", default: "var(--ink-internal-code-color)" },
    { suffix: "syntax-monospace-font-family", default: "var(--ink-internal-code-font-family)" },
    { suffix: "syntax-name-color", default: "#d19a66" },
    { suffix: "syntax-name-label-color", default: "#abb2bf" },
    { suffix: "syntax-name-property-color", default: "#96c0d8" },
    { suffix: "syntax-name-property-definition-color", default: "#e06c75" },
    { suffix: "syntax-name-variable-color", default: "#e06c75" },
    { suffix: "syntax-name-variable-definition-color", default: "#e5c07b" },
    { suffix: "syntax-name-variable-local-color", default: "#d19a66" },
    { suffix: "syntax-name-variable-special-color", default: "inherit" },
    { suffix: "syntax-number-color", default: "#d19a66" },
    { suffix: "syntax-operator-color", default: "#96c0d8" },
    { suffix: "syntax-processing-instruction-color", default: "#444444", light: "#bbbbbb" },
    { suffix: "syntax-punctuation-color", default: "#abb2bf" },
    { suffix: "syntax-strikethrough-color", default: "inherit" },
    { suffix: "syntax-strikethrough-text-decoration", default: "line-through" },
    { suffix: "syntax-string-color", default: "#98c379" },
    { suffix: "syntax-string-special-color", default: "inherit" },
    { suffix: "syntax-strong-color", default: "inherit" },
    { suffix: "syntax-strong-font-weight", default: "600" },
    { suffix: "syntax-url-color", default: "#aaaaaa", light: "#666666" }
  ];
  const isLight = !isDark(state.options.interface.appearance);
  return styles.map((style) => {
    const value = isLight && style.light ? style.light : style.default;
    return `--ink-internal-${style.suffix}: var(--ink-${style.suffix}, ${value});`;
  });
};

const dark = () => {
  return [
    EditorView.theme({}, { dark: true })
  ];
};
const light = () => {
  return [
    EditorView.theme({}, { dark: false })
  ];
};

class AttributionWidget extends WidgetType {
  constructor() {
    super();
  }
  eq() {
    return true;
  }
  toDOM() {
    const container = document.createElement("div");
    const text = container.appendChild(document.createElement("span"));
    const link = container.appendChild(document.createElement("a"));
    container.setAttribute("aria-hidden", "true");
    container.style.bottom = "0";
    container.style.fontSize = "0.75em";
    container.style.opacity = "0.5";
    container.style.position = "absolute";
    container.style.right = "0";
    text.innerText = "powered by ";
    link.href = "https://github.com/voracious/ink-mde";
    link.innerText = "ink-mde";
    link.rel = "noopener noreferrer";
    link.target = "_blank";
    link.style.color = "var(--ink-internal-color)";
    link.style.fontWeight = "600";
    link.style.textDecoration = "none";
    return container;
  }
}
const attributionTheme = EditorView.baseTheme({
  ".cm-content": {
    paddingBottom: "2em"
  }
});
const attributionField = StateField.define({
  create(state) {
    return RangeSet.of(decoration().range(state.doc.length));
  },
  update(_attributions, transaction) {
    return RangeSet.of(decoration().range(transaction.newDoc.length));
  },
  provide(field) {
    return EditorView.decorations.from(field);
  }
});
const decoration = () => Decoration.widget({
  widget: new AttributionWidget(),
  side: 1,
  block: true
});
const attribution = () => {
  return [
    attributionTheme,
    attributionField
  ];
};

class ImageWidget extends WidgetType {
  url;
  constructor({ url }) {
    super();
    this.url = url;
  }
  eq(imageWidget) {
    return imageWidget.url === this.url;
  }
  toDOM() {
    const container = document.createElement("div");
    const backdrop = container.appendChild(document.createElement("div"));
    const figure = backdrop.appendChild(document.createElement("figure"));
    const image = figure.appendChild(document.createElement("img"));
    container.setAttribute("aria-hidden", "true");
    container.className = "cm-image-container";
    backdrop.className = "cm-image-backdrop";
    figure.className = "cm-image-figure";
    image.className = "cm-image-img";
    image.src = this.url;
    container.style.paddingBottom = "0.5rem";
    container.style.paddingTop = "0.5rem";
    backdrop.classList.add("cm-image-backdrop");
    backdrop.style.borderRadius = "var(--ink-internal-border-radius)";
    backdrop.style.display = "flex";
    backdrop.style.alignItems = "center";
    backdrop.style.justifyContent = "center";
    backdrop.style.overflow = "hidden";
    backdrop.style.maxWidth = "100%";
    figure.style.margin = "0";
    image.style.display = "block";
    image.style.maxHeight = "var(--ink-internal-block-max-height)";
    image.style.maxWidth = "100%";
    image.style.width = "100%";
    return container;
  }
}
const images = () => {
  const imageRegex = /!\[.*?\]\((?<url>.*?)\)/;
  const imageDecoration = (imageWidgetParams) => Decoration.widget({
    widget: new ImageWidget(imageWidgetParams),
    side: -1,
    block: true
  });
  const decorate = (state) => {
    const widgets = [];
    syntaxTree(state).iterate({
      enter: ({ type, from, to }) => {
        if (type.name === "Image") {
          const result = imageRegex.exec(state.doc.sliceString(from, to));
          if (result && result.groups && result.groups.url)
            widgets.push(imageDecoration({ url: result.groups.url }).range(state.doc.lineAt(from).from));
        }
      }
    });
    return widgets.length > 0 ? RangeSet.of(widgets) : Decoration.none;
  };
  const imagesField = StateField.define({
    create(state) {
      return decorate(state);
    },
    update(images2, transaction) {
      if (transaction.docChanged)
        return decorate(transaction.state);
      return images2.map(transaction.changes);
    },
    provide(field) {
      return EditorView.decorations.from(field);
    }
  });
  return [
    imagesField
  ];
};

const spellcheck = () => {
  return EditorView.contentAttributes.of({
    spellcheck: "true"
  });
};

const buildVendor = (extension, options) => {
  const result = extension.resolver(options);
  return extension.compartment.of(result);
};
const buildVendors = (state) => {
  return state.extensions.map((extension) => {
    return buildVendor(extension, state.options);
  });
};
const buildVendorUpdate = (extension, options) => {
  const result = extension.resolver(options);
  return extension.compartment.reconfigure(result);
};
const buildVendorUpdates = (state) => {
  return state.extensions.map((extension) => {
    return buildVendorUpdate(extension, state.options);
  });
};
const create = (name) => {
  const compartment = new Compartment();
  const resolver = resolvers[name];
  return {
    compartment,
    name,
    resolver
  };
};
const createExtensions = () => {
  return [
    create(Extensions.Appearance),
    create(Extensions.Attribution),
    create(Extensions.Autocomplete),
    create(Extensions.Images),
    create(Extensions.ReadOnly),
    create(Extensions.Spellcheck),
    create(Extensions.Vim)
  ];
};
const resolvers = {
  appearance({ interface: { appearance } }) {
    if (appearance === Appearance.Dark)
      return dark();
    if (appearance === Appearance.Light)
      return light();
    return isAutoDark() ? dark() : light();
  },
  attribution(options) {
    return options.interface.attribution ? attribution() : [];
  },
  autocomplete(options) {
    return options.interface.autocomplete ? closeBrackets() : [];
  },
  images(options) {
    return options.interface.images ? images() : [];
  },
  readonly(options) {
    return options.interface.readonly ? EditorState.readOnly.of(true) : EditorState.readOnly.of(false);
  },
  spellcheck(options) {
    return options.interface.spellcheck ? spellcheck() : [];
  },
  vim(options) {
    return options.vim ? vim() : [];
  }
};

const blankState = () => {
  const options = {
    doc: "",
    files: {
      clipboard: false,
      dragAndDrop: false,
      handler: () => {
      },
      injectMarkup: true,
      types: ["image/*"]
    },
    hooks: {
      afterUpdate: () => {
      },
      beforeUpdate: () => {
      }
    },
    interface: {
      appearance: Appearance.Auto,
      attribution: true,
      autocomplete: false,
      images: false,
      readonly: false,
      spellcheck: true,
      toolbar: false
    },
    plugins: [],
    selections: [],
    toolbar: {
      bold: true,
      code: true,
      codeBlock: true,
      heading: true,
      image: true,
      italic: true,
      link: true,
      list: true,
      orderedList: true,
      quote: true,
      taskList: true,
      upload: false
    },
    vim: false
  };
  return {
    editor: {},
    extensions: createExtensions(),
    options,
    root: createElement(),
    target: createElement()
  };
};
const makeState$1 = (partialState) => {
  return override(blankState(), partialState);
};
const makeStore = (options, overrides = {}) => {
  const [state, setState] = createSignal(makeState$1({ ...overrides, options }));
  return [state, setState];
};

const toCodeMirror = (selections) => {
  const ranges = selections.map((selection) => {
    const range = SelectionRange.fromJSON({ anchor: selection.start, head: selection.end });
    return range;
  });
  return EditorSelection.create(ranges);
};
const toInk = (selection) => {
  const selections = selection.ranges.map((range) => {
    return {
      end: range.anchor < range.head ? range.head : range.anchor,
      start: range.head < range.anchor ? range.head : range.anchor
    };
  });
  return selections;
};

const defineConfig = (overrides) => {
  const defaults = {
    block: false,
    line: false,
    multiline: false,
    nodes: [],
    prefix: "",
    prefixStates: [],
    suffix: ""
  };
  return { ...defaults, ...overrides };
};
const formatting = {
  [Markup.Bold]: defineConfig({
    nodes: ["StrongEmphasis"],
    prefix: "**",
    suffix: "**"
  }),
  [Markup.Code]: defineConfig({
    nodes: ["InlineCode"],
    prefix: "`",
    suffix: "`"
  }),
  [Markup.CodeBlock]: defineConfig({
    block: true,
    nodes: ["FencedCode"],
    prefix: "```\n",
    suffix: "\n```"
  }),
  [Markup.Heading]: defineConfig({
    multiline: true,
    nodes: ["ATXHeading1", "ATXHeading2", "ATXHeading3", "ATXHeading4", "ATXHeading5", "ATXHeading6"],
    prefix: "# ",
    prefixStates: ["# ", "## ", "### ", "#### ", "##### ", "###### ", ""]
  }),
  [Markup.Image]: defineConfig({
    nodes: ["Image"],
    prefix: "![](",
    suffix: ")"
  }),
  [Markup.Italic]: defineConfig({
    nodes: ["Emphasis"],
    prefix: "*",
    suffix: "*"
  }),
  [Markup.Link]: defineConfig({
    nodes: ["Link"],
    prefix: "[](",
    suffix: ")"
  }),
  [Markup.OrderedList]: defineConfig({
    line: true,
    multiline: true,
    nodes: ["OrderedList"],
    prefix: "1. "
  }),
  [Markup.Quote]: defineConfig({
    line: true,
    multiline: true,
    nodes: ["Blockquote"],
    prefix: "> "
  }),
  [Markup.TaskList]: defineConfig({
    line: true,
    multiline: true,
    nodes: ["BulletList"],
    prefix: "- [ ] "
  }),
  [Markup.List]: defineConfig({
    line: true,
    multiline: true,
    nodes: ["BulletList"],
    prefix: "- "
  })
};
const getBlockSelection = ([state, _setState], selection) => {
  const { editor } = state();
  const startLine = editor.lineBlockAt(selection.start);
  const endLine = editor.lineBlockAt(selection.end);
  return { start: startLine.from, end: endLine.to };
};
const splitSelectionByLines = ([state, _setState], selection) => {
  const { editor } = state();
  let position = selection.start;
  const selections2 = [];
  while (position <= selection.end) {
    const line = editor.lineBlockAt(position);
    const start = Math.max(position, line.from);
    const end = Math.min(position, line.to);
    selections2.push({ start, end });
    position = line.to + 1;
  }
  return selections2;
};
const getInlineSelection = ([state, _setState], selection) => {
  const { editor } = state();
  const start = editor.state.wordAt(selection.start)?.from || selection.start;
  const end = editor.state.wordAt(selection.end)?.to || selection.end;
  return { start, end };
};
const getSelection = ([state, setState], userSelection) => {
  return userSelection || selections([state, setState]).pop() || { start: 0, end: 0 };
};
const getText = ([state, _setState], userSelection) => {
  const { editor } = state();
  return editor.state.sliceDoc(userSelection.start, userSelection.end);
};
const getNode = ([state, _setState], definition, selection) => {
  const selectionNodes = getNodes(state(), selection);
  return selectionNodes.find(({ type }) => definition.nodes.includes(type.name));
};
const getNodes = ({ editor: { state } }, selection) => {
  const types = [];
  syntaxTree(state).iterate({
    from: selection.start,
    to: selection.end,
    enter: ({ type, from, to }) => {
      types.push({ type, from, to });
    }
  });
  return types;
};
const unformat = ([state, setState], definition, selection) => {
  const text = getText([state, setState], selection);
  const sliceStart = definition.prefix.length;
  const sliceEnd = definition.suffix.length * -1 || text.length;
  const unformatted = text.slice(sliceStart, sliceEnd);
  insert([state, setState], unformatted, selection);
};
const formatBlock = ([state, setState], definition, selection) => {
  const blockSelection = getBlockSelection([state, setState], selection);
  const blockNode = getNode([state, setState], definition, blockSelection);
  if (blockNode) {
    const start = blockNode.from;
    const end = blockNode.to;
    unformat([state, setState], definition, { start, end });
  } else {
    const before = definition.prefix;
    const after = definition.suffix;
    wrap([state, setState], { before, after, selection: blockSelection });
  }
};
const formatMultiline = ([state, setState], definition, selection) => {
  const selections2 = splitSelectionByLines([state, setState], selection);
  selections2.forEach((lineSelection) => {
    formatLine([state, setState], definition, lineSelection);
  });
};
const formatLine = ([state, setState], definition, selection) => {
  const lineSelection = getBlockSelection([state, setState], selection);
  const lineNode = getNode([state, setState], definition, lineSelection);
  const hasPrefixStates = definition.prefixStates.length > 0;
  if (lineNode && hasPrefixStates) {
    definition.prefixStates.find((prefix, index) => {
      const start = lineNode.from;
      const end = lineNode.to;
      const text = getText([state, setState], { start, end });
      const isMatch = text.startsWith(prefix);
      if (isMatch)
        insert([state, setState], text.replace(new RegExp(`^${prefix}`), definition.prefixStates[index + 1]), { start, end });
      return isMatch;
    });
  } else if (lineNode) {
    const start = lineNode.from;
    const end = lineNode.to;
    unformat([state, setState], definition, { start, end });
  } else {
    const before = definition.prefix;
    const after = definition.suffix;
    wrap([state, setState], { before, after, selection: lineSelection });
  }
};
const formatInline = ([state, setState], definition, selection) => {
  const node = getNode([state, setState], definition, selection);
  if (node) {
    const start = node.from;
    const end = node.to;
    unformat([state, setState], definition, { start, end });
  } else {
    const { start, end } = getInlineSelection([state, setState], selection);
    const before = Array.isArray(definition.prefix) ? definition.prefix[0] : definition.prefix;
    const after = definition.suffix;
    wrap([state, setState], { before, after, selection: { start, end } });
  }
};
const format$1 = ([state, setState], formatType, userSelection) => {
  const definition = formatting[formatType];
  const selection = getSelection([state, setState], userSelection);
  if (definition.block) {
    formatBlock([state, setState], definition, selection);
  } else if (definition.multiline) {
    formatMultiline([state, setState], definition, selection);
  } else if (definition.line) {
    formatLine([state, setState], definition, selection);
  } else {
    formatInline([state, setState], definition, selection);
  }
};

const codeBlockSyntaxNodes = [
  "CodeBlock",
  "FencedCode",
  "HTMLBlock",
  "CommentBlock"
];
const codeBlockDecoration = Decoration.line({ attributes: { class: "cm-codeblock" } });
const codeBlockOpenDecoration = Decoration.line({ attributes: { class: "cm-codeblock-open" } });
const codeBlockCloseDecoration = Decoration.line({ attributes: { class: "cm-codeblock-close" } });
const codeDecoration = Decoration.mark({ attributes: { class: "cm-code" } });
const codeOpenDecoration = Decoration.mark({ attributes: { class: "cm-code cm-code-open" } });
const codeCloseDecoration = Decoration.mark({ attributes: { class: "cm-code cm-code-close" } });
const codeBlockPlugin = ViewPlugin.define((view) => {
  return {
    update: () => {
      return decorate(view);
    }
  };
}, { decorations: (plugin) => plugin.update() });
const decorate = (view) => {
  const builder = new RangeSetBuilder();
  const tree = syntaxTree(view.state);
  for (const visibleRange of view.visibleRanges) {
    for (let position = visibleRange.from; position < visibleRange.to; ) {
      const line = view.state.doc.lineAt(position);
      let inlineCode;
      tree.iterate({
        enter({ type, from, to }) {
          if (type.name !== "Document") {
            if (codeBlockSyntaxNodes.includes(type.name)) {
              builder.add(line.from, line.from, codeBlockDecoration);
              const openLine = view.state.doc.lineAt(from);
              const closeLine = view.state.doc.lineAt(to);
              if (openLine.number === line.number)
                builder.add(line.from, line.from, codeBlockOpenDecoration);
              if (closeLine.number === line.number)
                builder.add(line.from, line.from, codeBlockCloseDecoration);
              return false;
            } else if (type.name === "InlineCode") {
              inlineCode = { from, to, innerFrom: from, innerTo: to };
            } else if (type.name === "CodeMark") {
              if (from === inlineCode.from) {
                inlineCode.innerFrom = to;
                builder.add(from, to, codeOpenDecoration);
              } else if (to === inlineCode.to) {
                inlineCode.innerTo = from;
                builder.add(inlineCode.innerFrom, inlineCode.innerTo, codeDecoration);
                builder.add(from, to, codeCloseDecoration);
              }
            }
          }
        },
        from: line.from,
        to: line.to
      });
      position = line.to + 1;
    }
  }
  return builder.finish();
};
const code = () => {
  return [
    codeBlockPlugin
  ];
};

const inkClassExtensions = () => {
  return [
    EditorView.editorAttributes.of({
      class: "ink-mde-editor"
    }),
    EditorView.contentAttributes.of({
      class: "ink-mde-editor-content"
    })
  ];
};
const ink$1 = () => {
  return [
    ...inkClassExtensions()
  ];
};

const keyMaps = [
  {
    key: "Tab",
    run: ({ state, dispatch }) => {
      if (state.selection.ranges.some((r) => !r.empty))
        return indentMore({ state, dispatch });
      dispatch(state.update(state.replaceSelection("  "), {
        scrollIntoView: true,
        annotations: Transaction.userEvent.of("input")
      }));
      return true;
    },
    shift: indentLess
  }
];
const keymaps = () => {
  return keymap.of([
    ...keyMaps,
    ...defaultKeymap,
    ...historyKeymap
  ]);
};

const lineWrapping = () => {
  return EditorView.lineWrapping;
};

const theme = () => {
  const extension = syntaxHighlighting(HighlightStyle.define([
    {
      tag: tags.atom,
      color: "var(--ink-internal-syntax-atom-color)"
    },
    {
      tag: tags.meta,
      color: "var(--ink-internal-syntax-meta-color)"
    },
    {
      tag: tags.emphasis,
      color: "var(--ink-internal-syntax-emphasis-color)",
      fontStyle: "var(--ink-internal-syntax-emphasis-font-style)"
    },
    {
      tag: tags.strong,
      color: "var(--ink-internal-syntax-strong-color)",
      fontWeight: "var(--ink-internal-syntax-strong-font-weight)"
    },
    {
      tag: tags.strikethrough,
      color: "var(--ink-internal-syntax-strikethrough-color)",
      textDecoration: "var(--ink-internal-syntax-strikethrough-text-decoration)"
    },
    {
      tag: tags.comment,
      color: "var(--ink-internal-syntax-comment-color)",
      fontStyle: "var(--ink-internal-syntax-comment-font-style)"
    },
    {
      tag: tags.monospace,
      color: "var(--ink-internal-syntax-code-color)",
      fontFamily: "var(--ink-internal-syntax-code-font-family)"
    },
    {
      tag: tags.name,
      color: "var(--ink-internal-syntax-name-color)"
    },
    {
      tag: tags.labelName,
      color: "var(--ink-internal-syntax-name-label-color)"
    },
    {
      tag: tags.propertyName,
      color: "var(--ink-internal-syntax-name-property-color)"
    },
    {
      tag: tags.definition(tags.propertyName),
      color: "var(--ink-internal-syntax-name-property-definition-color)"
    },
    {
      tag: tags.variableName,
      color: "var(--ink-internal-syntax-name-variable-color)"
    },
    {
      tag: tags.definition(tags.variableName),
      color: "var(--ink-internal-syntax-name-variable-definition-color)"
    },
    {
      tag: tags.local(tags.variableName),
      color: "var(--ink-internal-syntax-name-variable-local-color)"
    },
    {
      tag: tags.special(tags.variableName),
      color: "var(--ink-internal-syntax-name-variable-special-color)"
    },
    {
      tag: tags.heading,
      color: "var(--ink-internal-syntax-heading-color)",
      fontWeight: "var(--ink-internal-syntax-heading-font-weight)"
    },
    {
      tag: tags.heading1,
      color: "var(--ink-internal-syntax-heading1-color)",
      fontSize: "var(--ink-internal-syntax-heading1-font-size)",
      fontWeight: "var(--ink-internal-syntax-heading1-font-weight)"
    },
    {
      tag: tags.heading2,
      color: "var(--ink-internal-syntax-heading2-color)",
      fontSize: "var(--ink-internal-syntax-heading2-font-size)",
      fontWeight: "var(--ink-internal-syntax-heading2-font-weight)"
    },
    {
      tag: tags.heading3,
      color: "var(--ink-internal-syntax-heading3-color)",
      fontSize: "var(--ink-internal-syntax-heading3-font-size)",
      fontWeight: "var(--ink-internal-syntax-heading3-font-weight)"
    },
    {
      tag: tags.heading4,
      color: "var(--ink-internal-syntax-heading4-color)",
      fontSize: "var(--ink-internal-syntax-heading4-font-size)",
      fontWeight: "var(--ink-internal-syntax-heading4-font-weight)"
    },
    {
      tag: tags.heading5,
      color: "var(--ink-internal-syntax-heading5-color)",
      fontSize: "var(--ink-internal-syntax-heading5-font-size)",
      fontWeight: "var(--ink-internal-syntax-heading5-font-weight)"
    },
    {
      tag: tags.heading6,
      color: "var(--ink-internal-syntax-heading6-color)",
      fontSize: "var(--ink-internal-syntax-heading6-font-size)",
      fontWeight: "var(--ink-internal-syntax-heading6-font-weight)"
    },
    {
      tag: tags.keyword,
      color: "var(--ink-internal-syntax-keyword-color)"
    },
    {
      tag: tags.number,
      color: "var(--ink-internal-syntax-number-color)"
    },
    {
      tag: tags.operator,
      color: "var(--ink-internal-syntax-operator-color)"
    },
    {
      tag: tags.punctuation,
      color: "var(--ink-internal-syntax-punctuation-color)"
    },
    {
      tag: tags.link,
      color: "var(--ink-internal-syntax-link-color)"
    },
    {
      tag: tags.url,
      color: "var(--ink-internal-syntax-url-color)"
    },
    {
      tag: tags.string,
      color: "var(--ink-internal-syntax-string-color)"
    },
    {
      tag: tags.special(tags.string),
      color: "var(--ink-internal-syntax-string-special-color)"
    },
    {
      tag: tags.processingInstruction,
      color: "var(--ink-internal-syntax-processing-instruction-color)"
    }
  ]));
  return [
    extension
  ];
};

const toVendorSelection = (selections) => {
  if (selections.length > 0)
    return toCodeMirror(selections);
};
const makeState = (state) => {
  const completions = state.options.plugins.flatMap((plugin) => plugin.type === PluginType.Completion ? plugin.value : []);
  const extensions = state.options.plugins.flatMap((plugin) => plugin.type === PluginType.Default ? plugin.value : []);
  const grammars = state.options.plugins.flatMap((plugin) => plugin.type === PluginType.Grammar ? plugin.value : []);
  return EditorState.create({
    doc: state.options.doc,
    selection: toVendorSelection(state.options.selections),
    extensions: [
      ...buildVendors(state),
      autocompletion({
        defaultKeymap: true,
        icons: false,
        override: completions,
        optionClass: () => "ink-tooltip-option"
      }),
      code(),
      history(),
      ink$1(),
      keymaps(),
      lineWrapping(),
      markdown({
        base: markdownLanguage,
        codeLanguages: languages,
        extensions: grammars
      }),
      theme(),
      ...extensions
    ]
  });
};

const destroy = ([state]) => {
  const { editor } = state();
  editor.destroy();
};
const doc = ([state, setState]) => {
  return getDoc([state, setState]);
};
const focus = ([state]) => {
  const { editor } = state();
  if (!editor.hasFocus)
    editor.focus();
};
const format = ([state, setState], type, selection) => {
  return format$1([state, setState], type, selection);
};
const getDoc = ([state]) => {
  const { editor } = state();
  return editor.state.sliceDoc();
};
const insert = ([state, setState], text, selection, updateSelection = false) => {
  const { editor } = state();
  let start = selection?.start;
  let end = selection?.end || selection?.start;
  if (typeof start === "undefined") {
    const current = selections([state, setState]).pop();
    start = current.start;
    end = current.end;
  }
  const updates = { changes: { from: start, to: end, insert: text } };
  if (updateSelection) {
    const anchor = start === end ? start + text.length : start;
    const head = start === end ? start + text.length : start + text.length;
    Object.assign(updates, { selection: { anchor, head } });
  }
  editor.dispatch(editor.state.update(updates));
};
const load = ([state, setState], doc2) => {
  setState(override(state(), { options: { doc: doc2 } }));
  state().editor.setState(makeState(state()));
};
const makeInstance = (store) => {
  return {
    destroy: destroy.bind(void 0, store),
    doc: doc.bind(void 0, store),
    focus: focus.bind(void 0, store),
    insert: insert.bind(void 0, store),
    load: load.bind(void 0, store),
    options: options.bind(void 0, store),
    reconfigure: reconfigure.bind(void 0, store),
    select: select.bind(void 0, store),
    selections: selections.bind(void 0, store),
    update: update.bind(void 0, store),
    wrap: wrap.bind(void 0, store)
  };
};
const options = ([state]) => {
  return state().options;
};
const reconfigure = ([state, setState], options2) => {
  const effects = buildVendorUpdates(setState(override(state(), { options: options2 })));
  state().editor.dispatch({
    effects
  });
};
const select = ([state], selections2) => {
  const { editor } = state();
  editor.dispatch(editor.state.update({
    selection: toCodeMirror(selections2)
  }));
};
const selections = ([state]) => {
  const { editor } = state();
  return toInk(editor.state.selection);
};
const update = ([state], doc2) => {
  const { editor } = state();
  editor.dispatch(editor.state.update({
    changes: {
      from: 0,
      to: editor.state.doc.length,
      insert: doc2
    }
  }));
};
const wrap = ([state, setState], { after, before, selection: userSelection }) => {
  const { editor } = state();
  const selection = userSelection || selections([state, setState]).pop() || { start: 0, end: 0 };
  const text = editor.state.sliceDoc(selection.start, selection.end);
  insert([state, setState], `${before}${text}${after}`, selection);
  select([state, setState], [{ start: selection.start + before.length, end: selection.end + before.length }]);
};

const styles$2 = ".ink-drop-zone {\n  align-items: center;\n  background-color: rgba(0, 0, 0, 0.5);\n  color: var(--ink-internal-color);\n  display: flex;\n  inset: 0;\n  justify-content: center;\n  position: var(--ink-internal-modal-position);\n  z-index: 100;\n}\n\n.ink-drop-zone:not(.visible) {\n  display: none;\n}\n\n.ink-drop-zone-modal {\n  background-color: var(--ink-internal-block-background-color);\n  border-radius: var(--ink-internal-border-radius);\n  box-sizing: border-box;\n  height: 100%;\n  max-height: 20rem;\n  max-width: 40rem;\n  padding: 1rem;\n  position: relative;\n  width: 100%;\n}\n\n.ink-drop-zone-hide {\n  cursor: pointer;\n  height: 1.75rem;\n  position: absolute;\n  right: 0.25rem;\n  top: 0.25rem;\n  width: 1.75rem;\n}\n\n.ink-drop-zone-hide svg {\n  background-color: var(--ink-internal-block-background-color);\n}\n\n.ink-drop-zone-droppable-area {\n  align-items: center;\n  border: 0.2rem dashed var(--ink-internal-color);\n  border-radius: 0.125rem;\n  box-sizing: border-box;\n  display: flex;\n  flex-direction: column;\n  font-size: 1.25em;\n  gap: 1rem;\n  height: 100%;\n  justify-content: center;\n  padding: 1rem;\n  text-align: center;\n}\n\n.ink-drop-zone-file-preview {\n  align-items: center;\n  display: flex;\n  flex-wrap: wrap;\n  gap: 0.5rem;\n  max-width: 25.5rem;\n}\n\n.ink-drop-zone-file-preview-image {\n  border: 0.125rem solid #222;\n  border-radius: 0.125rem;\n  box-sizing: border-box;\n  height: 6rem;\n  object-fit: cover;\n  padding: 0.5rem;\n  width: 6rem;\n}\n";

const _tmpl$$5 = ["<span", ">uploading files...</span>"],
      _tmpl$2$1 = ["<div", " class=\"", "\"><style>", "</style><div class=\"ink-drop-zone-modal\"><div class=\"ink-drop-zone-droppable-area\"><div class=\"ink-drop-zone-file-preview\">", "</div><!--#-->", "<!--/--></div><div class=\"ink-drop-zone-hide\"><svg xmlns=\"http://www.w3.org/2000/svg\" fill=\"none\" viewBox=\"0 0 24 24\" stroke=\"currentColor\"><path stroke-linecap=\"round\" stroke-linejoin=\"round\" stroke-width=\"2\" d=\"M10 14l2-2m0 0l2-2m-2 2l-2-2m2 2l2 2m7-2a9 9 0 11-18 0 9 9 0 0118 0z\"></path></svg></div></div></div>"],
      _tmpl$3$1 = ["<img", " class=\"ink-drop-zone-file-preview-image\"", ">"],
      _tmpl$4$1 = ["<span", ">drop files here</span>"];
const DropZone = () => {
  const [depth, setDepth] = createSignal(0);
  const [files, setFiles] = createSignal([]);
  const [isLoading, setIsLoading] = createSignal(false);
  const [isVisible, setIsVisible] = createSignal(false);
  const [state, setState] = useStore();

  const onDragEnter = event => {
    if (state().options.files.dragAndDrop) {
      event.preventDefault();
      setDepth(depth() + 1);
      setIsVisible(true);
    }
  };

  const onDragLeave = event => {
    if (state().options.files.dragAndDrop) {
      event.preventDefault();
      setDepth(depth() - 1);
      if (depth() === 0) setIsVisible(false);
    }
  };

  const onDragOver = event => {
    if (state().options.files.dragAndDrop) {
      event.preventDefault();
      setIsVisible(true);
    }
  };

  const onDrop = event => {
    if (state().options.files.dragAndDrop) {
      event.preventDefault();
      setDepth(0);
      setIsVisible(false);
    }
  };

  const onPaste = event => {
    if (state().options.files.clipboard) {
      event.preventDefault();
      const transfer = event.clipboardData;
      if (transfer?.files && transfer.files.length > 0) uploadFiles(transfer.files);
    }
  };

  const uploadFiles = userFiles => {
    Array.from(userFiles).forEach(file => {
      setFiles([...files(), file]);
    });
    setIsLoading(true);
    setIsVisible(true);
    Promise.resolve(state().options.files.handler(userFiles)).then(url => {
      if (state().options.files.injectMarkup && url) {
        const markup = `![](${url})`;
        insert([state, setState], markup);
      }
    }).finally(() => {
      setDepth(0);
      setIsLoading(false);
      setIsVisible(false);
      setFiles([]);
    });
  };

  onMount(() => {
    document.addEventListener('dragenter', onDragEnter);
    document.addEventListener('dragleave', onDragLeave);
    document.addEventListener('dragover', onDragOver);
    document.addEventListener('drop', onDrop);
    state().root.addEventListener('paste', onPaste);
  });
  onCleanup(() => {
    document.removeEventListener('dragenter', onDragEnter);
    document.removeEventListener('dragleave', onDragLeave);
    document.removeEventListener('dragover', onDragOver);
    document.removeEventListener('drop', onDrop);
    state().root.removeEventListener('paste', onPaste);
  });
  return ssr(_tmpl$2$1, ssrHydrationKey(), `ink-drop-zone ${isVisible() ? "visible" : ""}`, escape(styles$2), escape(createComponent(For, {
    get each() {
      return files().slice(0, 8);
    },

    children: file => ssr(_tmpl$3$1, ssrHydrationKey(), ssrAttribute("alt", escape(file.name, true), false) + ssrAttribute("src", escape(URL.createObjectURL(file), true), false))
  })), escape(createComponent(Show, {
    get when() {
      return isLoading();
    },

    get fallback() {
      return ssr(_tmpl$4$1, ssrHydrationKey());
    },

    get children() {
      return ssr(_tmpl$$5, ssrHydrationKey());
    }

  })));
};

const _tmpl$$4 = ["<div", " class=\"cm-editor\"><div class=\"cm-scroller\"><div class=\"cm-content\"", "><div class=\"cm-line\"><br></div></div></div></div>"];
const Editor = () => {
  // Needed for tree-shaking purposes.
  {
    return ssr(_tmpl$$4, ssrHydrationKey(), ssrAttribute("contenteditable", escape(true, true), false));
  }
};

const _tmpl$$3 = ["<button", " class=\"ink-button\">", "</button>"];
const Button = props => {
  return ssr(_tmpl$$3, ssrHydrationKey(), escape(props.children));
};

const styles$1 = ".ink .ink-toolbar {\n  background-color: var(--ink-internal-block-background-color);\n  border-radius: var(--ink-internal-border-radius);\n  bottom: 0.25rem;\n  color: inherit;\n  display: flex;\n  flex-shrink: 0;\n  gap: 2rem;\n  overflow: auto;\n  padding: 0.25rem;\n  position: sticky;\n  top: 0.25rem;\n  z-index: 10;\n}\n\n.ink .ink-toolbar-group {\n  display: flex;\n}\n\n.ink .ink-toolbar .ink-button {\n  align-items: center;\n  background: none;\n  border: none;\n  border-radius: var(--ink-internal-border-radius);\n  color: inherit;\n  cursor: pointer;\n  display: flex;\n  height: 2.25rem;\n  justify-content: center;\n  padding: 0.4rem;\n  width: 2.25rem;\n}\n\n.ink .ink-toolbar .ink-button:hover {\n  background-color: var(--ink-internal-block-background-color-on-hover);\n}\n\n.ink .ink-toolbar .ink-button > * {\n  align-items: center;\n  display: flex;\n  height: 100%;\n}\n";

const _tmpl$$2 = ["<svg", " viewBox=\"0 0 20 20\" fill=\"none\" stroke=\"currentColor\" stroke-miterlimit=\"5\" stroke-linecap=\"round\" stroke-linejoin=\"round\"><path d=\"M6 4V10M6 16V10M6 10H14M14 10V4M14 10V16\"></path></svg>"],
      _tmpl$2 = ["<svg", " viewBox=\"0 0 20 20\" fill=\"none\" stroke=\"currentColor\" stroke-width=\"1.5\" stroke-miterlimit=\"5\" stroke-linecap=\"round\" stroke-linejoin=\"round\"><path d=\"M6.5 10H10.5C12.1569 10 13.5 11.3431 13.5 13C13.5 14.6569 12.1569 16 10.5 16H6.5V4H9.5C11.1569 4 12.5 5.34315 12.5 7C12.5 8.65686 11.1569 10 9.5 10\"></path></svg>"],
      _tmpl$3 = ["<svg", " viewBox=\"0 0 20 20\" fill=\"none\" stroke=\"currentColor\" stroke-miterlimit=\"5\" stroke-linecap=\"round\" stroke-linejoin=\"round\"><path d=\"M11 4L9 16M13 4H9M7 16H11\"></path></svg>"],
      _tmpl$4 = ["<svg", " viewBox=\"0 0 20 20\" fill=\"none\" stroke=\"currentColor\" stroke-miterlimit=\"5\" stroke-linecap=\"round\" stroke-linejoin=\"round\"><path d=\"M2.00257 16H17.9955M2.00055 4H18M7 10H18.0659M2 8.5V11.4999C2.4 11.5 2.5 11.5 2.5 11.5V11V10.5M4 8.5V11.4999H4.5V11V10.5\"></path></svg>"],
      _tmpl$5 = ["<svg", " viewBox=\"0 0 20 20\" fill=\"none\" stroke=\"currentColor\" stroke-miterlimit=\"5\" stroke-linecap=\"round\" stroke-linejoin=\"round\"><path d=\"M13 4L7 16\"></path><path d=\"M5 7L2 10L5 13\"></path><path d=\"M15 7L18 10L15 13\"></path></svg>"],
      _tmpl$6 = ["<svg", " viewBox=\"0 0 20 20\" fill=\"none\" stroke=\"currentColor\" stroke-miterlimit=\"5\" stroke-linecap=\"round\" stroke-linejoin=\"round\"><path d=\"M7 4L8 6\"></path></svg>"],
      _tmpl$7 = ["<svg", " viewBox=\"0 0 20 20\" fill=\"none\" stroke=\"currentColor\" stroke-miterlimit=\"5\" stroke-linecap=\"round\" stroke-linejoin=\"round\"><path d=\"M7 16H17.8294\"></path><path d=\"M2 16H4\"></path><path d=\"M7 10H17.8294\"></path><path d=\"M2 10H4\"></path><path d=\"M7 4H17.8294\"></path><path d=\"M2 4H4\"></path></svg>"],
      _tmpl$8 = ["<svg", " viewBox=\"0 0 20 20\" fill=\"none\" stroke=\"currentColor\" stroke-miterlimit=\"5\" stroke-linecap=\"round\" stroke-linejoin=\"round\"><path d=\"M7 16H18\"></path><path d=\"M2 17.0242C2.48314 17.7569 3.94052 17.6154 3.99486 16.7919C4.05315 15.9169 3.1975 16.0044 2.99496 16.0044M2.0023 14.9758C2.48544 14.2431 3.94282 14.3846 3.99716 15.2081C4.05545 16.0831 3.1998 16.0002 2.99726 16.0002\"></path><path d=\"M7 10H18\"></path><path d=\"M2.00501 11.5H4M2.00193 8.97562C2.48449 8.24319 3.9401 8.38467 3.99437 9.20777C4.05259 10.0825 2.04342 10.5788 2 11.4996\"></path><path d=\"M7 4H18\"></path><path d=\"M2 5.5H4M2.99713 5.49952V2.5L2.215 2.93501\"></path></svg>"],
      _tmpl$9 = ["<svg", " viewBox=\"0 0 20 20\" fill=\"none\" stroke=\"currentColor\" stroke-miterlimit=\"5\" stroke-linecap=\"round\" stroke-linejoin=\"round\"><path d=\"M7 16H17.8294\"></path><path d=\"M5 15L3 17L2 16\"></path><path d=\"M7 10H17.8294\"></path><path d=\"M5 9L3 11L2 10\"></path><path d=\"M7 4H17.8294\"></path><path d=\"M5 3L3 5L2 4\"></path></svg>"],
      _tmpl$10 = ["<svg", " viewBox=\"0 0 20 20\" fill=\"none\" stroke=\"currentColor\" stroke-miterlimit=\"5\" stroke-linecap=\"round\" stroke-linejoin=\"round\"><path d=\"M9.12127 10.881C10.02 11.78 11.5237 11.7349 12.4771 10.7813L15.2546 8.00302C16.2079 7.04937 16.253 5.54521 15.3542 4.6462C14.4555 3.74719 12.9512 3.79174 11.9979 4.74539L10.3437 6.40007M10.8787 9.11903C9.97997 8.22002 8.47626 8.26509 7.52288 9.21874L4.74545 11.997C3.79208 12.9506 3.74701 14.4548 4.64577 15.3538C5.54452 16.2528 7.04876 16.2083 8.00213 15.2546L9.65633 13.5999\"></path></svg>"],
      _tmpl$11 = ["<svg", " viewBox=\"0 0 20 20\" fill=\"none\" stroke=\"currentColor\" stroke-miterlimit=\"5\" stroke-linecap=\"round\" stroke-linejoin=\"round\"><rect x=\"2\" y=\"4\" width=\"16\" height=\"12\" rx=\"1\"></rect><path d=\"M7.42659 7.67597L13.7751 13.8831M2.00208 12.9778L7.42844 7.67175\"></path><path d=\"M11.9119 12.0599L14.484 9.54443L17.9973 12.9785\"></path><path d=\"M10.9989 7.95832C11.551 7.95832 11.9986 7.52072 11.9986 6.98092C11.9986 6.44113 11.551 6.00354 10.9989 6.00354C10.4468 6.00354 9.99921 6.44113 9.99921 6.98092C9.99921 7.52072 10.4468 7.95832 10.9989 7.95832Z\"></path></svg>"],
      _tmpl$12 = ["<svg", " viewBox=\"0 0 20 20\" fill=\"none\" stroke=\"currentColor\" stroke-miterlimit=\"5\" stroke-linecap=\"round\" stroke-linejoin=\"round\"><path d=\"M10 13V4M10 4L13 7M10 4L7 7\"></path><path d=\"M2 13V15C2 15.5523 2.44772 16 3 16H17C17.5523 16 18 15.5523 18 15V13\"></path></svg>"],
      _tmpl$13 = ["<input", " style=\"", "\" type=\"file\">"],
      _tmpl$14 = ["<div", " class=\"ink-toolbar\"><style>", "</style><div class=\"ink-toolbar-group\"><!--#-->", "<!--/--><!--#-->", "<!--/--><!--#-->", "<!--/--></div><div class=\"ink-toolbar-group\"><!--#-->", "<!--/--><!--#-->", "<!--/--><!--#-->", "<!--/--></div><div class=\"ink-toolbar-group\"><!--#-->", "<!--/--><!--#-->", "<!--/--><!--#-->", "<!--/--></div><div class=\"ink-toolbar-group\"><!--#-->", "<!--/--><!--#-->", "<!--/--><!--#-->", "<!--/--></div></div>"];
const Toolbar = () => {
  const [state, setState] = useStore();
  const [uploader, setUploader] = createSignal();

  const formatAs = type => {
    format([state, setState], type);
    focus([state, setState]);
  };

  const uploadClickHandler = () => {
    uploader()?.click();
  };

  return ssr(_tmpl$14, ssrHydrationKey(), escape(styles$1), escape(createComponent(Show, {
    get when() {
      return state().options.toolbar.heading;
    },

    get children() {
      return createComponent(Button, {
        onclick: () => formatAs(Markup.Heading),

        get children() {
          return ssr(_tmpl$$2, ssrHydrationKey());
        }

      });
    }

  })), escape(createComponent(Show, {
    get when() {
      return state().options.toolbar.bold;
    },

    get children() {
      return createComponent(Button, {
        onclick: () => formatAs(Markup.Bold),

        get children() {
          return ssr(_tmpl$2, ssrHydrationKey());
        }

      });
    }

  })), escape(createComponent(Show, {
    get when() {
      return state().options.toolbar.italic;
    },

    get children() {
      return createComponent(Button, {
        onclick: () => formatAs(Markup.Italic),

        get children() {
          return ssr(_tmpl$3, ssrHydrationKey());
        }

      });
    }

  })), escape(createComponent(Show, {
    get when() {
      return state().options.toolbar.quote;
    },

    get children() {
      return createComponent(Button, {
        onclick: () => formatAs(Markup.Quote),

        get children() {
          return ssr(_tmpl$4, ssrHydrationKey());
        }

      });
    }

  })), escape(createComponent(Show, {
    get when() {
      return state().options.toolbar.codeBlock;
    },

    get children() {
      return createComponent(Button, {
        onclick: () => formatAs(Markup.CodeBlock),

        get children() {
          return ssr(_tmpl$5, ssrHydrationKey());
        }

      });
    }

  })), escape(createComponent(Show, {
    get when() {
      return state().options.toolbar.code;
    },

    get children() {
      return createComponent(Button, {
        onclick: () => formatAs(Markup.Code),

        get children() {
          return ssr(_tmpl$6, ssrHydrationKey());
        }

      });
    }

  })), escape(createComponent(Show, {
    get when() {
      return state().options.toolbar.list;
    },

    get children() {
      return createComponent(Button, {
        onclick: () => formatAs(Markup.List),

        get children() {
          return ssr(_tmpl$7, ssrHydrationKey());
        }

      });
    }

  })), escape(createComponent(Show, {
    get when() {
      return state().options.toolbar.orderedList;
    },

    get children() {
      return createComponent(Button, {
        onclick: () => formatAs(Markup.OrderedList),

        get children() {
          return ssr(_tmpl$8, ssrHydrationKey());
        }

      });
    }

  })), escape(createComponent(Show, {
    get when() {
      return state().options.toolbar.taskList;
    },

    get children() {
      return createComponent(Button, {
        onclick: () => formatAs(Markup.TaskList),

        get children() {
          return ssr(_tmpl$9, ssrHydrationKey());
        }

      });
    }

  })), escape(createComponent(Show, {
    get when() {
      return state().options.toolbar.link;
    },

    get children() {
      return createComponent(Button, {
        onclick: () => formatAs(Markup.Link),

        get children() {
          return ssr(_tmpl$10, ssrHydrationKey());
        }

      });
    }

  })), escape(createComponent(Show, {
    get when() {
      return state().options.toolbar.image;
    },

    get children() {
      return createComponent(Button, {
        onclick: () => formatAs(Markup.Image),

        get children() {
          return ssr(_tmpl$11, ssrHydrationKey());
        }

      });
    }

  })), escape(createComponent(Show, {
    get when() {
      return state().options.toolbar.upload;
    },

    get children() {
      return createComponent(Button, {
        onclick: uploadClickHandler,

        get children() {
          return [ssr(_tmpl$12, ssrHydrationKey()), ssr(_tmpl$13, ssrHydrationKey(), "display:" + "none")];
        }

      });
    }

  })));
};

const styles = ".ink {\n  color: var(--ink-internal-color, inherit);\n  display: flex;\n  flex-direction: var(--ink-internal-flex-direction, column);\n  gap: 1rem;\n  padding: var(--ink-internal-editor-padding, 0);\n}\n\n.ink .cm-tooltip {\n  background-color: var(--ink-internal-block-background-color);\n  border-radius: var(--ink-internal-border-radius);\n  font-family: inherit;\n  padding: 0.25rem;\n}\n\n.ink .cm-tooltip.cm-tooltip-autocomplete ul {\n  font-family: inherit;\n}\n\n.ink .cm-tooltip.cm-tooltip-autocomplete ul li.ink-tooltip-option {\n  border-radius: var(--ink-internal-border-radius);\n  padding: 0.25rem;\n}\n\n.ink .cm-tooltip.cm-tooltip-autocomplete ul li.ink-tooltip-option[aria-selected] {\n  background-color: rgba(150, 150, 150, 0.25);\n}\n\n.ink .cm-completionLabel {\n  font-family: inherit;\n}\n\n.ink-mde, .ink-mde * {\n  box-sizing: border-box;\n}\n\n.ink-mde .cm-editor {\n  display: flex;\n  flex-direction: column;\n  position: relative;\n}\n\n.ink-mde .cm-scroller {\n  align-items: flex-start;\n  display: flex;\n  font-family: var(--ink-internal-font-family);\n  font-size: var(--ink-internal-editor-font-size);\n  line-height: var(--ink-internal-editor-line-height);\n  overflow-x: auto;\n  position: relative;\n}\n\n.ink-mde .cm-content {\n  display: block;\n  flex-grow: 2;\n  flex-shrink: 0;\n  margin: 0;\n  outline: none;\n  padding: 0;\n  white-space: pre;\n  word-wrap: normal;\n}\n\n.ink-mde .cm-lineWrapping {\n  flex-shrink: 1;\n  overflow-wrap: anywhere;\n  white-space: break-spaces;\n  word-break: break-word;\n}\n\n.ink-mde .cm-line {\n  font-family: var(--ink-internal-font-family);\n  padding: 0;\n}\n\n.ink-mde .cm-line.cm-codeblock {\n  background-color: var(--ink-internal-block-background-color);\n  font-family: var(--ink-internal-code-font-family);\n  padding: 0 var(--ink-internal-block-padding);\n}\n\n.ink-mde .cm-line.cm-codeblock.cm-codeblock-open {\n  border-radius: var(--ink-internal-border-radius) var(--ink-internal-border-radius) 0 0;\n  padding-top: var(--ink-internal-block-padding);\n}\n\n.ink-mde .cm-line.cm-codeblock.cm-codeblock-close {\n  border-radius: 0 0 var(--ink-internal-border-radius) var(--ink-internal-border-radius);\n  padding-bottom: var(--ink-internal-block-padding);\n}\n\n.ink-mde .cm-line .cm-code {\n  background-color: var(--ink-internal-block-background-color);\n  font-family: var(--ink-internal-code-font-family);\n  padding: var(--ink-internal-inline-padding) 0;\n}\n\n.ink-mde .cm-line .cm-code.cm-code-open {\n  border-radius: var(--ink-internal-border-radius) 0 0 var(--ink-internal-border-radius);\n  padding-left: var(--ink-internal-inline-padding);\n}\n\n.ink-mde .cm-line .cm-code.cm-code-close {\n  border-radius: 0 var(--ink-internal-border-radius) var(--ink-internal-border-radius) 0;\n  padding-right: var(--ink-internal-inline-padding);\n}\n\n.ink-mde .cm-image-backdrop {\n  background-color: var(--ink-internal-block-background-color);\n}\n";

const _tmpl$$1 = ["<style", ">", "</style>"];
const Styles = () => {
  const [state] = useStore();
  const [vars, setVars] = createSignal(makeVars(state()));
  createEffect(() => {
    setVars(makeVars(state()));
  });
  onMount(() => {
    const mediaQueryList = window.matchMedia('(prefers-color-scheme: dark)');

    const listener = _event => {
      const {
        editor,
        root
      } = state();

      if (root.isConnected) {
        const effects = buildVendorUpdates(state());
        editor.dispatch({
          effects
        });
        setVars(makeVars(state()));
      } else {
        mediaQueryList.removeEventListener('change', listener);
      }
    };

    mediaQueryList.addEventListener('change', listener);
  });
  return ssr(_tmpl$$1, ssrHydrationKey(), `.ink {\n  ${escape(vars().join('\n  '))}\n}\n${escape(styles)}`);
};

const _tmpl$ = ["<div", " class=\"ink ink-mde\" ", "><!--#-->", "<!--/--><!--#-->", "<!--/--><!--#-->", "<!--/--><!--#-->", "<!--/--></div>"];
const Root = () => {
  const [state, setState] = useStore();

  return ssr(_tmpl$, ssrHydrationKey(), ssrSpread(getHydrationMarkerProps(), false, true), escape(createComponent(Styles, {})), escape(createComponent(Show, {
    get when() {
      return state().options.files.clipboard || state().options.files.dragAndDrop;
    },

    get children() {
      return createComponent(DropZone, {});
    }

  })), escape(createComponent(Show, {
    get when() {
      return state().options.interface.toolbar;
    },

    get children() {
      return createComponent(Toolbar, {});
    }

  })), escape(createComponent(Editor, {})));
};

const AppContext = createContext([() => blankState(), value => typeof value === 'function' ? value(blankState()) : value]);

const AppProvider = props => {
  return createComponent(AppContext.Provider, {
    get value() {
      return props.store;
    },

    get children() {
      return props.children;
    }

  });
};

const useStore = () => {
  return useContext(AppContext);
};
const App = props => {
  return createComponent(AppProvider, {
    get store() {
      return props.store;
    },

    get children() {
      return createComponent(Root, {
        get store() {
          return props.store;
        }

      });
    }

  });
};

const defineOptions = options => {
  return options;
};
const hydrate = (target, options = {}) => {
  const store = makeStore(options);

  return makeInstance(store);
};
const ink = (target, options = {}) => {
  const hasHydrationMarker = !!target.querySelector(HYDRATION_MARKER_SELECTOR);

  if (hasHydrationMarker) {
    return hydrate(target, options);
  }

  return render(target, options);
};
const render = (target, options = {}) => {
  const store = makeStore(options);

  return makeInstance(store);
};
const renderToString = (options = {}) => {
  const store = makeStore(options); // Needed for tree-shaking purposes.

  return renderToString$1(() => createComponent(App, {
    store: store
  }));
};
const solidPrepareForHydration = () => {
  // @ts-expect-error Todo: This is a third-party vendor script.
  // eslint-disable-next-line
  let e, t;
  e = window._$HY || (window._$HY = {
    events: [],
    completed: new WeakSet(),
    r: {}
  }), t = e => e && e.hasAttribute && (e.hasAttribute("data-hk") ? e : t(e.host && e.host instanceof Node ? e.host : e.parentNode)), ['click', 'input'].forEach(o => document.addEventListener(o, o => {
    let s = o.composedPath && o.composedPath()[0] || o.target,
        a = t(s);
    a && !e.completed.has(a) && e.events.push([a, o]);
  })), e.init = (t, o) => {
    e.r[t] = [new Promise((e, t) => o = e), o];
  }, e.set = (t, o, s) => {
    (s = e.r[t]) && s[1](o), e.r[t] = [o];
  }, e.unset = t => {
    delete e.r[t];
  }, e.load = (t, o) => {
    if (o = e.r[t]) return o[0];
  };
};

export { Appearance, Extensions, Markup, PluginType, ink as default, defineOptions, hydrate, ink, render, renderToString, solidPrepareForHydration };
//# sourceMappingURL=index.js.map
